/* This file was generated by syncqt. */
#ifndef QT_QTSPATIALAUDIO_VERSION_H
#define QT_QTSPATIALAUDIO_VERSION_H

#define QTSPATIALAUDIO_VERSION_STR "6.10.2"

#define QTSPATIALAUDIO_VERSION 0x0601002

#endif // QT_QTSPATIALAUDIO_VERSION_H
